% Finite Element Methods in linear structural analysis %
% Study work                                           %
% Task 1                                               %
% Student: Enes Siljak                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Function that forms element stiffness matrix 
% is in the file ElementStiffnessMatrix.m
%
% a, b tenth of last and second last number of matr.number
% length unit cm; force unit N;
%
% In this task it is requested that Symbolic math toolbox 
% have to be used. That's why, I'm using length l, E and A as 
% symbolic variables and calculating simbolically global stiffness matrix. 

a = 0.7;
b = 0.8;
syms l;

%  coordOfNode = nodes(nodeNumb, 1 = X[2 = Y])
syms nodes;
nodes = l*[0 b
0 0
2 b
2 0
2+2*a b
2+2*a 0
4+2*a b
4+2*a 0];

%  numbOfNodeOnElement = elements(elNumb, 1 = nodei[2 = nodej])
elements =[1 2
1 3
1 4
2 4
3 4
3 5
4 5
4 6
5 6
5 7
5 8
6 8
7 8];

syms E; 
syms A; 

numbOfElem = 13;

% equationNumber = dofs[nodeNumb, 1 = displ X [2 = displ Y][-1 = fixed, no
% displ/equation]
% (equationNumber or address in KGlob)

dofs = [1 2
3 -1
4 5
6 7
8 9
10 11
12 13
-1 -1];

KGlob = sym(zeros(13,13));

syms Ke;

for i = 1:numbOfElem
  
  ni = elements(i, 1);
  nj = elements(i, 2);

  x1 = nodes(ni, 1);
  y1 = nodes(ni, 2);
  
  x2 = nodes(nj, 1);
  y2 = nodes(nj, 2);
  
  Ke = ElementStiffnessMatrix(x1, y1, x2, y2, E, A);
  
  addr(1) = dofs(ni, 1);
  addr(2) = dofs(ni, 2);
  addr(3) = dofs(nj, 1);
  addr(4) = dofs(nj, 2);
  
    for j = 1:4
        for k = 1:4
            if ( (addr(j) ~= -1) && (addr(k) ~= -1) )
                KGlob( addr(j), addr(k) ) = KGlob( addr(j), addr(k) ) + Ke(j, k);
            end
        end % for k
    end % for j

end %for i through all elements  

'Global Stiffness Matrix:'
pretty(KGlob)

l = 100.0;
A = 1.0;
E = 2.1*10^7;

KGlobNum = subs(KGlob)

RGlob = zeros(13);
RGlob(9) = -100;

'Global load vector [N]:'
RGlob(1:13,1)

u = KGlobNum \ RGlob;


% extracting displacements of nodes from u to nodesDispl
nodesDispl = zeros(8, 2);

for i = 1:8
    addr(1) = dofs(i, 1);
    addr(2) = dofs(i, 2);
    
    if addr(1) ~= -1 
        nodesDispl (i, 1) = u( addr(1) );
    end
    if addr(2) ~= -1 
        nodesDispl (i, 2) = u( addr(2) );
    end     
end
'Displacements [cm]:'
nodesDispl

% absolute position of nodes after deformation
nodesDispl = subs(nodes) + nodesDispl;

% forces in elements F = EA * dL/L
ForceOfEl = zeros(13);
for i = 1:13
    
  ni = elements(i, 1);
  nj = elements(i, 2);

  % undeformed position of nodes
  x1 = subs(nodes(ni, 1));
  y1 = subs(nodes(ni, 2));
  x2 = subs(nodes(nj, 1));
  y2 = subs(nodes(nj, 2));
  
  % undeformed length
  Len = sqrt((x2-x1)^2 + (y2-y1)^2);
  
  % deformed position of nodes
  x1 = nodesDispl(ni, 1);
  y1 = nodesDispl(ni, 2);
  x2 = nodesDispl(nj, 1);
  y2 = nodesDispl(nj, 2);  
  
  % deformed length
  dLLen = sqrt((x2-x1)^2 + (y2-y1)^2);
  
  dL = dLLen - Len;
  
  ForceOfEl(i) = E*A*dL/Len;
    
end

'Forces [N]:'
ForceOfEl(1:13,1)







